"use strict";
const fs = require("fs");
const json_parse_helpfulerror_1 = require("json-parse-helpfulerror");
const bluebird_1 = require("bluebird");
const rimraf = require("rimraf");
const readFileAsync = bluebird_1.Promise.promisify(fs.readFile);
const writeFileAsync = bluebird_1.Promise.promisify(fs.writeFile);
function readText(file) {
    return readFileAsync(file, "utf8");
}
exports.readText = readText;
function readBytes(file) {
    return readFileAsync(file);
}
exports.readBytes = readBytes;
function writeFile(path, data) {
    return writeFileAsync(path, data);
}
exports.writeFile = writeFile;
function parseJsonFile(file) {
    return readText(file).
        then(it => parseJson(it, file));
}
exports.parseJsonFile = parseJsonFile;
function parseJson(data, path) {
    try {
        return json_parse_helpfulerror_1.parse(data);
    }
    catch (e) {
        if (e instanceof SyntaxError) {
            throw new Error("Cannot parse '" + path + "': " + e.message);
        }
        else {
            throw e;
        }
    }
}
exports.parseJson = parseJson;
function deleteFile(path, ignoreIfNotExists = false) {
    return new bluebird_1.Promise((resolve, reject) => {
        fs.unlink(path, it => it == null || (ignoreIfNotExists && it.code === "ENOENT") ? resolve(null) : reject(it));
    });
}
exports.deleteFile = deleteFile;
function deleteDirectory(path) {
    return new bluebird_1.Promise((resolve, reject) => {
        rimraf(path, { glob: false }, error => error == null ? resolve(null) : reject(error));
    });
}
exports.deleteDirectory = deleteDirectory;
function renameFile(oldPath, newPath) {
    return new bluebird_1.Promise((resolve, reject) => {
        fs.rename(oldPath, newPath, error => error == null ? resolve(newPath) : reject(error));
    });
}
exports.renameFile = renameFile;
const statFileAsync = bluebird_1.Promise.promisify(fs.stat);
function stat(path) {
    return statFileAsync(path);
}
exports.stat = stat;
//# sourceMappingURL=promisifed-fs.js.map